// SphereColorTableRendererDX11.cpp: SphereColorTableRendererDX11 クラスのインプリメンテーション
//
//////////////////////////////////////////////////////////////////////

#include "SphereColorTableRendererDX11.h"

#include "atomic_color.h"

#define _USE_MATH_DEFINES
#include <math.h>
#include <string.h>
#include <algorithm>



//#pragma warning(disable : 4996)

extern size_t g_max_vram_size;

#define		ATOM_RADIUS		(0.25)


SphereColorTableRendererDX11::SphereColorTableRendererDX11(ID3D11Device* pD3DDevice, ID3D11DeviceContext* pD3DDeviceContext, CommonShader* comShader) :
	renderer(pD3DDevice, pD3DDeviceContext, comShader),
	m_vram_size(0),
	m_vbo_vertex(NULL),
	m_vbo_indexes(NULL),	
	m_geometry_sphere(NULL),
	m_CB_color_table(pD3DDevice, pD3DDeviceContext, 1),
	m_CB_range(pD3DDevice, pD3DDeviceContext, 2)
{
	m_prev_vis = VISUAL_SETTING{ 0 };
	

	InitShader();
}


SphereColorTableRendererDX11::~SphereColorTableRendererDX11() {

	delete[] m_vbo_vertex;
	delete[] m_vbo_indexes;

	if (m_geometry_sphere) delete m_geometry_sphere;
	
}


void SphereColorTableRendererDX11::InitShader() {
	TCHAR filepath[MAX_PATH];
	D3D11_INPUT_ELEMENT_DESC layout[] =
	{
		{ "POSITION", 0, DXGI_FORMAT_R32G32B32_FLOAT, 0, 0, D3D11_INPUT_PER_VERTEX_DATA, 0 },
		{ "NORMAL", 0, DXGI_FORMAT_R32G32B32_FLOAT, 0, 12, D3D11_INPUT_PER_VERTEX_DATA, 0 },
		//{ "COLOR", 0, DXGI_FORMAT_R8G8B8A8_UNORM, 0, 24, D3D11_INPUT_PER_VERTEX_DATA, 0 },		
		{ "INS_POS", 0, DXGI_FORMAT_R32G32B32A32_FLOAT, 1, 0, D3D11_INPUT_PER_INSTANCE_DATA, 1 },
		
		
	};
	UINT numElements = ARRAYSIZE(layout);
	GetAttachedFileName(filepath, _T("vs_inst_color_table.cso"));
	renderer.LoadVertexShader(filepath, "main", "vs_4_0", layout, numElements);


	//Pixel Shader-----------------------------------------
	GetAttachedFileName(filepath, _T("ps_color.cso"));
	renderer.LoadPixelShader(filepath, "main", "ps_4_0");
}


//Instanceを描画//
void SphereColorTableRendererDX11::Draw(int num_particles, const vec3f* r, const float* color_table, const int num_colors, float height_min, float height_max, const VISUAL_SETTING& vis) {

	if (num_particles <= 0) return;
	if (r == NULL) return;
	
	//vboのセット
	if ((vis.atom_poly != m_prev_vis.atom_poly) || (vis.atom_radius != m_prev_vis.atom_radius)) {
		//ポリゴンモデルの初期化

		if (m_geometry_sphere) delete m_geometry_sphere;
		m_geometry_sphere = new GeometrySphere(vis.atom_radius, vis.atom_poly, vis.atom_poly * 2);
		CreateSphere();
	}
	

	//原子球の描画//
	if (vis.atom) {

		renderer.UpdateVertexBuffer(m_vbo_vertex, m_vbo_vcount, sizeof(VERTEX_F3F3));	//頂点データをVRAMに転送
		renderer.UpdateIndexBuffer(m_vbo_indexes, m_vbo_icount, sizeof(int));	//インデックスデータをVRAMに転送

		m_CB_color_table.Update(color_table, num_colors, sizeof(float)*4);
		
		m_CB_range.Update(&(VERTEX_F4(height_min, height_max- height_min,(float)(num_colors - 1),0.0)), 1, sizeof(float)*4);
		//m_CB_range.Update(&(VERTEX_F4(1.0, 0.0, 1.0, 1.0)), 1, sizeof(float) * 4);
		
		//一度に書く頂点数の計算//
		size_t num_draw_once = std::min<size_t>((g_max_vram_size / (sizeof(VERTEX_F3UI1))), num_particles);
		
		size_t i_start = 0;
		while (i_start < num_particles) {

			if (i_start + num_draw_once > num_particles) { num_draw_once = num_particles - i_start; };
			
			//原子の位置と色のセット//
			m_instances.clear();			
			for (int i = i_start, const i_end = i_start + num_draw_once; i < i_end; ++i) {
				m_instances.push_back(VERTEX_F4(r[i], r[i].z));				
			}
			UINT valid_count = m_instances.size();
			
			if (!m_instances.empty()) {
				renderer.UpdateInstanceBuffer((LPVOID)&(m_instances[0]), valid_count, sizeof(VERTEX_F3UI1));
				//renderer.Draw(m_vbo_icount, D3D_PRIMITIVE_TOPOLOGY_TRIANGLELIST, valid_count);
				renderer.BeginDraw(D3D_PRIMITIVE_TOPOLOGY_TRIANGLELIST);
				m_CB_color_table.AttachVS();
				m_CB_range.AttachVS();
				renderer.EndDraw(m_vbo_icount, valid_count);

			}

			i_start += num_draw_once;
		}
		
	}


	m_prev_vis = vis;

}



void SphereColorTableRendererDX11::CreateSphere() {

	const int vbo_v_count = m_geometry_sphere->GetNumVertexes();
	const int vbo_i_count = m_geometry_sphere->GetNumIndexes();
	//vboサイズ
	int need_vram_sz = vbo_v_count * sizeof(VERTEX_F3F3) + vbo_i_count * sizeof(int);

	if (m_vram_size < need_vram_sz) {
		m_vram_size = need_vram_sz;

		delete[] m_vbo_vertex;
		delete[] m_vbo_indexes;
		m_vbo_vertex = new VERTEX_F3F3[vbo_v_count];
		m_vbo_indexes = new int[vbo_i_count];
	}
	
	m_vbo_vcount = 0;
	m_vbo_icount = 0;

	//原子(球)のセット//
	const uint32_t color = 0xFFFFFFFF;
		
	m_geometry_sphere->SetSphere(vec3f(0.0,0.0,0.0), 
				m_vbo_vcount,	//頂点番号のoffset
				m_vbo_vertex,
				m_vbo_indexes);


			m_vbo_vcount += vbo_v_count;
			m_vbo_icount += vbo_i_count;

}


CylinderColorTableRendererDX11::CylinderColorTableRendererDX11(ID3D11Device* pD3DDevice, ID3D11DeviceContext* pD3DDeviceContext, CommonShader* comShader) :
	renderer(pD3DDevice, pD3DDeviceContext, comShader),
	m_vram_size(0),
	m_vbo_vertex(NULL),
	m_vbo_indexes(NULL),
	m_visual_bond(VISUAL_BOND_PIPE),
	m_geometry_bond(NULL)
{

	memset(&m_prev_vis, 0, sizeof(VISUAL_SETTING));


	InitShader();
}


CylinderColorTableRendererDX11::~CylinderColorTableRendererDX11() {

	delete[] m_vbo_vertex;
	delete[] m_vbo_indexes;


	if (m_geometry_bond) delete m_geometry_bond;




}

void CylinderColorTableRendererDX11::InitShader() {
	TCHAR filepath[MAX_PATH];
	D3D11_INPUT_ELEMENT_DESC layout[] =
	{
		{ "POSITION", 0, DXGI_FORMAT_R32G32B32_FLOAT, 0, 0, D3D11_INPUT_PER_VERTEX_DATA, 0 },
		{ "NORMAL", 0, DXGI_FORMAT_R32G32B32_FLOAT, 0, 12, D3D11_INPUT_PER_VERTEX_DATA, 0 },
		{ "COLOR", 0, DXGI_FORMAT_R8G8B8A8_UNORM, 0, 24, D3D11_INPUT_PER_VERTEX_DATA, 0 },

	};
	UINT numElements = ARRAYSIZE(layout);
	GetAttachedFileName(filepath, _T("vs_color.cso"));
	renderer.LoadVertexShader(filepath, "main", "vs_4_0", layout, numElements);


	//Pixel Shader-----------------------------------------
	GetAttachedFileName(filepath, _T("ps_color.cso"));
	renderer.LoadPixelShader(filepath, "main", "ps_4_0");
}

//VBOの設定
void CylinderColorTableRendererDX11::Draw(int num_pairs, const VECTOR_IJ* bonds, const vec3f* r, const int* color_indexes, const uint32_t* color_table, const VISUAL_SETTING& vis){

	if (num_pairs == 0) return;
	if (bonds == NULL) return;
	

	if ((vis.bond_poly != m_prev_vis.bond_poly) || (vis.bond != m_prev_vis.bond)) {
		//ポリゴンモデルの初期化

		if (m_geometry_bond) delete m_geometry_bond;
		m_geometry_bond = new GeometryCylinder(vis.atom_radius * 0.25, vis.bond_poly);

	}




	//ボンドの描画//
	
	size_t num_draw_once = CalcNumDrawOnce(m_geometry_bond, g_max_vram_size);


	size_t i_start = 0;
	while (i_start < num_pairs) {

		if (i_start + num_draw_once > num_pairs) { num_draw_once = num_pairs - i_start; };


		CreateBond(i_start, num_draw_once, bonds, r, color_indexes, color_table);


		if (m_vbo_icount) {

			renderer.UpdateVertexBuffer(m_vbo_vertex, m_vbo_vcount, sizeof(VERTEX_F3F3UI1));	//頂点データをVRAMに転送
			renderer.UpdateIndexBuffer(m_vbo_indexes, m_vbo_icount, sizeof(int));	//インデックスデータをVRAMに転送


			renderer.DrawIndexed(m_vbo_icount, D3D_PRIMITIVE_TOPOLOGY_TRIANGLELIST);
		}

		i_start += num_draw_once;
	}



	m_prev_vis = vis;
}

void CylinderColorTableRendererDX11::CreateBond(int start, int count, const VECTOR_IJ* bonds, const vec3f* r, const int* color_indexes, const uint32_t* color_table) {

	const int geo_v_count = m_geometry_bond->GetNumVertexes();
	const int geo_i_count = m_geometry_bond->GetNumIndexes();

	//vboサイズ
	int vbo_v_count = count * geo_v_count;
	int vbo_i_count = count * geo_i_count;
	int need_vram_sz = vbo_v_count * sizeof(VERTEX_F3F3UI1) + vbo_i_count * sizeof(int);

	
	if (m_vram_size < need_vram_sz) {
		m_vram_size = need_vram_sz;

		delete[] m_vbo_vertex;
		delete[] m_vbo_indexes;
		m_vbo_vertex = new VERTEX_F3F3UI1[vbo_v_count];
		m_vbo_indexes = new int[vbo_i_count];
	}



	m_vbo_vcount = 0;
	m_vbo_icount = 0;


	//ボンドのセット
	//const VECTOR_IJ* b = bond->bvector;
	const int i_end = start + count;
	for (int i = start; i < i_end; ++i) {
		
		uint32_t color_i = color_table[color_indexes[bonds[i].i]];
		uint32_t color_j = color_table[color_indexes[bonds[i].j]];
		if ((color_i & color_j) & 0xFF000000) {	//完全透過は表示しない//

			m_geometry_bond->SetBond(r[bonds[i].i], bonds[i].v,
				color_i,
				color_j,
				m_vbo_vcount,
				m_vbo_vertex + m_vbo_vcount,
				m_vbo_indexes + m_vbo_icount);

			m_vbo_vcount += geo_v_count;
			m_vbo_icount += geo_i_count;
		}
	}


}

